/***************************************************************************
 *            int2.c
 *
 *  Tue Aug 15 19:59:38 2006
 *  Copyright  2006  sharandac
 *  Email sharandac[at]snafu.de
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <avr/interrupt.h>
#include <stdio.h>
#include "int2.h"

INT2_CALLBACK_FUNC INT2_CallbackFunc[ MAX_INT2_CALLBACKS ];

void INT2_init(void)
	{
		
		cli();
		
		unsigned char i;
		
		// Alle Callbackeinträge löschen
		for ( i = 0 ; i < MAX_INT2_CALLBACKS ; i++ ) INT2_CallbackFunc[i] = NULL;
			
		GICR |= 0x20; // Interruptfreigabe für INT2
	
		sei();
	}
			
		

ISR(INT2_vect)       /* veraltet: SIGNAL(SIG_INTERRUPT0) */
	{
 		unsigned char tmp_sreg;  	// temporaerer Speicher fuer das Statusregister
   		tmp_sreg = SREG;   			// Statusregister (also auch das I-Flag darin) sichern
  		cli();             			// Interrupts global deaktivieren

		unsigned char i;

		for ( i = 0 ; i < MAX_INT2_CALLBACKS ; i++ ) if ( INT2_CallbackFunc[i] != NULL ) INT2_CallbackFunc[i]();
		
		SREG = tmp_sreg;    		// Status-Register wieder herstellen 
                      				// somit auch das I-Flag auf gesicherten Zustand setzen
	}
	
unsigned char INT2_RegisterCallbackFunction( INT2_CALLBACK_FUNC pFunc )
	{
		unsigned char i;
		
		for ( i = 0 ; i < MAX_INT2_CALLBACKS ; i++ ) 
		{
			if ( INT2_CallbackFunc[i] == pFunc )
				return TRUE;
		}
		
		for ( i = 0 ; i < MAX_INT2_CALLBACKS ; i++ )
		{
			if ( INT2_CallbackFunc[i] == NULL )
			{
				INT2_CallbackFunc[i] = pFunc;
				return TRUE;
			}
		}
		return FALSE;
	}

unsigned char INT2_RemoveCallbackFunction( INT2_CALLBACK_FUNC pFunc )
	{
		unsigned char i;
		
		for ( i = 0 ; i < MAX_INT2_CALLBACKS ; i++ )
		{
			if ( INT2_CallbackFunc[i] == pFunc )
			{
				INT2_CallbackFunc[i] = NULL;
				return TRUE;
			}
		}
		return FALSE;
	}
