/*! \file "arp.c" \brief Die Arp-Funktionlitaet */
//***************************************************************************
//*            arp.c
//*
//*  Mon Aug 28 22:31:14 2006
//*  Copyright  2006  sharandac
//*  Email sharandac(at)snafu.de
//****************************************************************************/
///	\ingroup network
///	\defgroup ARP ARP-Funktionen (arp.c)
///	\code #include "arp.h" \endcode
///	\code #include "ip.h" \endcode
///	\code #include "ethernet.h" \endcode
///	\par Uebersicht
//****************************************************************************/
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
//@{
#include <avr/pgmspace.h>
#include <stdio.h>
#include <stdlib.h>
#include "arp.h"
#include "ethernet.h"
#include "enc28j60.h"
#include "ip.h"
#include "../hardware/uart.h"
#include "../hardware/timer1.h"

struct ARP_TABLE ARPtable[ MAX_ARPTABLE_ENTRYS ];
struct ARP_TABLE *ARP_table;

void arp( unsigned int packet_lenght, unsigned char *ethernetbuffer)
	{
		unsigned char i;
		
		struct ETH_header *ETH_packet;
		ETH_packet = (struct ETH_header *) ethernetbuffer; 
		struct ARP_header *ARP_packet;
		ARP_packet = (struct ARP_header *) &ethernetbuffer[ETHERNET_HEADER_LENGTH];
		switch ( ARP_packet->ARP_Opcode ) {
			
			case 0x0100:		// check mal ob das für mich ist !
								if ( ARP_packet->ARP_destIP != myIP ) return ;
								// wenn ja fang mal an die antwort zusammen zu basteln
								ARP_packet->ARP_Opcode = 0x0200;
								// mac und ip des senders in ziel kopieren
								for ( i = 0; i < 10; i++ )ARP_packet->ARP_destMac[i] = ARP_packet->ARP_sourceMac[i]; // MAC und IP umkopieren
								// meine mac und ip als absender einsetzen
								for ( i = 0; i < 6 ; i++ )ARP_packet->ARP_sourceMac[i] = mymac[i]; // MAC einsetzen
								ARP_packet->ARP_sourceIP = myIP ; // IP einsetzen
								// sourceMAC in destMAC eintragen und meine MAC in sourceMAC kopieren
								for( i = 0 ; i < 6 ; i++){	
										ETH_packet->ETH_destMac[i] = ETH_packet->ETH_sourceMac[i];	
										ETH_packet->ETH_sourceMac[i] = mymac[i]; }
								sendEthernetframe( packet_lenght, ethernetbuffer);
								break;
			case 0x0200:		// check mal ob das für mich ist !
								ARP_table = &ARPtable[0];
								ARP_table->IP = ARP_packet->ARP_sourceIP;
								for ( i = 0 ; i < 6 ; i++ ) ARP_table->MAC[i] = ARP_packet->ARP_sourceMac[i];
								break;
		}
	}

/*!\brief Ermittelt für eine IP-Adresse die MAC-Adresse.
 * \param IP			Die Ip-Adresse von der die MAC-Adresse ermittelt werden soll.
 * \param MACbuffer		Zeiger auf de Speicherbereich von 6Byte wo die MAC-Adresse abgelegt wird.
 * \return				Die MAC-Adresse im Puffer.
 * \retval ARP_ANSWER	Die MAC anfrage war erfolgrich.
 * \retval NO_ARP_ANSWER Die MAC-anfrage ist Fehlgeschlagen.
 */
unsigned int GetIP2MAC( unsigned long IP, unsigned char * MACbuffer )
	{

		// unsigned char mymac[6] = { ENC28J60_MAC0,ENC28J60_MAC1,ENC28J60_MAC2,ENC28J60_MAC3,ENC28J60_MAC4,ENC28J60_MAC5 };
		unsigned int i;
		// Arp-request senden und schaun ob ip vorhanden

		ARP_table = &ARPtable[0];
		
/*		if ( ARP_table->IP == IP )
		{
			for( i = 0 ; i < 6 ; i++ ) MACbuffer[i] = ARP_table->MAC[i];
			return( ARP_ANSWER );
		} */

		ARP_table->IP = 0;
		
		unsigned char * ethernetbuffer;
		ethernetbuffer = (unsigned char*) __builtin_alloca (( size_t ) ETHERNET_HEADER_LENGTH + ARP_HEADER_LENGHT );
		
		struct ETH_header *ETH_packet;
		ETH_packet = (struct ETH_header *) ethernetbuffer; 
		struct ARP_header *ARP_packet;
		ARP_packet = (struct ARP_header *) &ethernetbuffer[ETHERNET_HEADER_LENGTH];

		// Hardware Type Ethernet
		ARP_packet->HWtype = 0x0100;
		// Protocoltype = IP
		ARP_packet->Protocoltype = 0x0008;
		// Hardware adresse size 6
		ARP_packet->HWsize = 0x06;
		// Protocolsize 4
		ARP_packet->Protocolsize = 0x4;
		// Opcode für request
		ARP_packet->ARP_Opcode = 0x0100;
		// mac auf 0 setzen
		for ( i = 0; i < 6; i++ )ARP_packet->ARP_destMac[i] = 0x00;
		// IP die abgefragt werden soll
		ARP_packet->ARP_destIP = IP;
		// meine mac und ip als absender einsetzen
		for ( i = 0; i < 6 ; i++ )ARP_packet->ARP_sourceMac[i] = mymac[i]; // MAC einsetzen
		ARP_packet->ARP_sourceIP = myIP ; // IP einsetzen
		// sourceMAC in destMAC in Ethernetframe eintragen

		for( i = 0 ; i < 6 ; i++){	
				ETH_packet->ETH_destMac[i] = 0xff;	
				ETH_packet->ETH_sourceMac[i] = mymac[i]; }
		ETH_packet->ETH_typefield = 0x0608;
				
		sendEthernetframe( ETHERNET_HEADER_LENGTH + ARP_HEADER_LENGHT , ethernetbuffer);				

		unsigned char timer;
		
		timer = timer1_msec_RegisterCoundowntimer();
		timer1_msec_SetCountdownTimer( timer , 10);

		while ( 1 )
		{
			if (  ( ARP_table->IP == IP ) && ( timer1_msec_GetCountdownTimer( timer ) != 0 ) )
			{
				for( i = 0 ; i < 6 ; i++ ) MACbuffer[i] = ARP_table->MAC[i];
				timer1_msec_ReleaseCountdownTimer( timer );
				return( ARP_ANSWER );
			}
			if ( timer1_msec_GetCountdownTimer( timer ) == 0 ) 
			{
				timer1_msec_ReleaseCountdownTimer( timer );
				return( NO_ARP_ANSWER );
			}
		}
	}
//@}
